#!/usr/bin/perl -w

use strict;

use SDL;
use SDL::App;
use SDL::Event;
use SDL::FontTool;
use SDL::Color;

my $app = SDL::App->new(-depth => 24);

my @entries = ("Start", "Enter Password", "Exit");

my $currently_selected = 0;

my @common_options = (
        -ttfont => "architxt.ttf",
        #-ttfont => "fkr_area51life.ttf",
        -size => 40,
    );

my $font = 
    SDL::FontTool->new(
        -normal => 1, 
        @common_options,
        -fg => $SDL::Color::black,
        -bg => $SDL::Color::black,
    );

my $font_bold =
    SDL::FontTool->new(
        -bold => 1, 
        @common_options,
        -fg => $SDL::Color::red,
        -bg => $SDL::Color::red,
    );

$app->fill(0, $SDL::Color::white);

for(my $i=0;$i<scalar(@entries);$i++)
{
    my $f;
    if ($i == $currently_selected)
    {
        $f = $font_bold;        
    }
    else
    {
        $f = $font;
    }
    $f->print($app,0, $i*50,$entries[$i]);
}

sub switch_to_a_different_entry
{
    my $different_entry = shift;
    
    my $cs = $currently_selected;
    $app->fill(SDL::Rect->new(-x => 0, -y => $cs*50, -width => 640, -height => 50), $SDL::Color::white);
    $font->print($app, 0, $currently_selected*50, $entries[$currently_selected]);

    $cs = $currently_selected = $different_entry;
    $app->fill(SDL::Rect->new(-x => 0, -y => $cs*50, -width => 640, -height => 50), $SDL::Color::white);            
    $font_bold->print($app, 0, $currently_selected*50, $entries[$currently_selected]);

}

$app->flip();

my ($event, $run_game);

$event = SDL::Event->new();

$run_game = 1;

sub menu_loop
{
    my $verdict;
    while ($run_game)
    {
        $event->pump();
        $event->poll();
        my $etype = $event->type();
        if ($etype eq SDL_QUIT())
        {
            last;
        }
        if ($etype eq SDL_KEYUP())
        {
            my $key = $event->key_sym();
            if ($key == SDLK_UP)
            {
                &switch_to_a_different_entry(($currently_selected == 0) ? $#entries : $currently_selected-1);
                $app->sync();
            }
            elsif ($key == SDLK_DOWN)
            {
                &switch_to_a_different_entry(($currently_selected == $#entries) ? 0 : $currently_selected+1);
                $app->sync();
            }
            elsif ($key == SDLK_RETURN)
            {
                my $cs = $currently_selected;
                my $e = $entries[$cs];
                if ($e eq "Start")
                {
                    $verdict =  { 'verdict' => 'start', 'board' => 1 };
                    $run_game = 0;
                }
                elsif ($e eq "Enter Password")
                {
                    $verdict = { 'verdict' => 'enter_password'};
                    $run_game = 0;
                }
                elsif ($e eq "Exit")
                {
                    $run_game = 0;
                }
            }
        }
        if (! $run_game)
        {
            next;
        }
        $event->wait();
    }
    return $verdict;
}

my $verdict = &menu_loop();

